from collections import namedtuple
import logging
import os

from configobj import ConfigObj
from validate import Validator

_LOG = logging.getLogger(__name__)


def _env_override():
    """Try to find SNAP_NETWORK_PATH env var"""

    try:
        return os.environ['SNAP_NETWORK_PATH']
    except:
        return None


def user_network_path():
    return _env_override() or os.path.expanduser(os.path.join('~', '.snap', 'networks.cfg'))


SCNetwork = namedtuple('SCNetwork', ['channel', 'net_id', 'encryption_type', 'encryption_key', 'feature_bits',
                                     'default_group', 'default_ttl', 'default_delay',
                                     'default_cb_retries', 'default_cb_timeout'])


class Network(object):

    _spec = [s.strip() for s in """
    channel = integer(min=0, max=16, default=4)
    net_id = string(min=0, max=6, default='0x1c2c')
    encryption_type = option('None', 'Basic', 'AES-128', default='None')
    encryption_key = string(min=0, max=16, default='')
    feature_bits = string(min=0, max=6, default='0x0100')
    default_group = integer(default=1)
    default_ttl = integer(default=5)
    default_delay = integer(default=50)
    default_cb_retries = integer(default=3)
    default_cb_timeout = integer(default=2)
    """.split('\n')]

    def __init__(self, name='default'):
        self.name = name
        self.channel = 4
        self.net_id = '0x1c2c'
        self.encryption_type = 'None'
        self.encryption_key = ''
        self.feature_bits = '0x0100'
        self.default_group = 1
        self.default_ttl = 5
        self.default_delay = 50
        self.default_cb_retries = 3
        self.default_cb_timeout = 2

    def __repr__(self):
        return "%s(%s: c=%d,n=%s,e=%s,f=%s, mcast_defaults=(%d, %d, %d), cb_defaults=(%d, %d))" % (self.__class__.__name__,
                                                                                                   self.name, self.channel,
                                                                                                   self.net_id,
                                                                                                   self.encryption_type,
                                                                                                   self.feature_bits,
                                                                                                   self.default_group,
                                                                                                   self.default_ttl,
                                                                                                   self.default_delay,
                                                                                                   self.default_cb_retries,
                                                                                                   self.default_cb_timeout)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and self.__dict__ == other.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    @staticmethod
    def default():
        return Network()

    @staticmethod
    def list():
        user_cfg = ConfigObj(user_network_path())
        return user_cfg.sections

    @staticmethod
    def delete(name):
        user_cfg = ConfigObj(user_network_path())
        user_cfg.pop(name)
        user_cfg.write()

    @classmethod
    def load(cls, name):
        network = cls(name)
        _LOG.debug("Reading Network: %s", name)
        v = Validator()
        cfg = ConfigObj(ConfigObj(user_network_path())[name], configspec=cls._spec)
        if cfg.validate(v):
            vars(network).update(**cfg)

        return network

    def save(self):
        user_cfg = ConfigObj(user_network_path(), write_empty_values=True)
        name = self.name.strip()
        if not name:
            raise ValueError("Network must have a name")

        user_cfg[name] = vars(self)
        # Don't write the name as a value in the section
        del user_cfg[name]['name']

        user_cfg.write()

    def as_sc_params(self):
        """Converts Network to SNAP Connect-ready parameters"""

        crypto_type = 0
        if self.encryption_type.lower() == "aes-128":
            crypto_type = 1
        elif self.encryption_type.lower() == "basic":
            crypto_type = 2

        return SCNetwork(self.channel, int(self.net_id, 16),
                         crypto_type, self.encryption_key,
                         int(self.feature_bits, 16),
                         self.default_group, self.default_ttl, self.default_delay,
                         self.default_cb_retries, self.default_cb_timeout)
