from collections import namedtuple
import binascii
import logging
import os
import re

from configobj import ConfigObj
from validate import Validator

_LOG = logging.getLogger(__name__)


def _env_override():
    """Try to find SNAP_PROFILE_PATH env var"""

    try:
        return os.environ['SNAP_PROFILE_PATH']
    except:
        return None


def user_profile_path():
    return _env_override() or os.path.expanduser(os.path.join('~', '.snap', 'profiles.cfg'))


class Profile(object):

    def __init__(self, name="default"):
        self.name = name

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self.name)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and self.__dict__ == other.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    @staticmethod
    def list():
        user_cfg = ConfigObj(user_profile_path())
        return user_cfg.sections

    @staticmethod
    def delete(name):
        user_cfg = ConfigObj(user_profile_path())
        user_cfg.pop(name)
        user_cfg.write()

    @classmethod
    def default(cls):
        return cls()

    @staticmethod
    def load(name):
        _LOG.debug("Reading Profile: %s", name)

        # So...this is awkward. Need to read enough of the profile
        # to find out what kind it is, then switch on that

        raw_cfg = ConfigObj(user_profile_path())[name]

        _LOG.debug("Raw Config: %r", raw_cfg)

        # sigh...
        if raw_cfg['kind'] == 'serial':
            cls = SerialProfile
        elif raw_cfg['kind'] == 'tcp':
            cls = TcpProfile
        elif raw_cfg['kind'] == 'mqtt':
            cls = MqttProfile

        _LOG.debug("Profile Class: %r", cls)

        profile = cls(name)
        cfg = ConfigObj(raw_cfg, configspec=cls._spec, write_empty_values=True)

        _LOG.debug("Config: %r", cfg)

        if cfg.validate(Validator()):
            vars(profile).update(**cfg)
        else:
            raise Exception("Validation failed!")

        return profile

    def save(self):
        user_cfg = ConfigObj(user_profile_path(), write_empty_values=True)
        name = self.name.strip()
        if not name:
            raise ValueError("Profile must have a name")

        user_cfg[name] = vars(self)
        # Don't write the name as a value in the section
        del user_cfg[name]['name']

        user_cfg.write()


SCSerialParams = namedtuple('SCSerialParams', ['type', 'port', 'address'])
SCTcpParams = namedtuple('SCTcpParams', ['host', 'port', 'ssl', 'username', 'password', 'address'])


class SerialProfile(Profile):

    _spec = [s.strip() for s in """
    kind = option('serial')
    device = string(default='/dev/snap1')
    address = string(min=0, max=6, default='')
    network = string(default='default')
    """.split('\n')]

    com_re = re.compile(r'^COM([0-9]+)$')  # Serial on Windows
    usb_re = re.compile(r'^USB([0-9]+)$')  # SS100 - USB
    ss_re = re.compile(r'^SnapStick([0-9]+)$')  # SS200 - USB
    dev_re = re.compile(r'^/dev/(snap|tty\.usbserial|ttyUSB|ttyS|tty\.SLAB_USBtoUART).*$')  # Serial on Posix
    addr_re = re.compile(r'^[0-9A-Fa-f]{6}$')

    def __init__(self, name='default'):
        super(SerialProfile, self).__init__(name)

        self.kind = "serial"
        self.device = "/dev/snap1"
        self.address = ""
        self.network = "default"

    def validate(self):
        return (self.kind == "serial" and
                (self.com_re.match(self.device) or
                 self.usb_re.match(self.device) or
                 self.ss_re.match(self.device) or
                 self.dev_re.match(self.device)) and
                (self.addr == "" or self.addr_re.match(self.addr)) is not None)

    def as_sc_params(self):
        """Translate from Profile to SC Params"""

        # Turn our device string into what SC expects
        usb_match = self.usb_re.match(self.device)
        if usb_match:
            # USB SNAPStick 100, aka old paddleboard.
            # Why aren't you in VCP mode???
            serial_type = 2
            # USB0 -> port 0
            serial_port = int(usb_match.groups()[0])
        else:
            ss_match = self.ss_re.match(self.device)
            if ss_match:
                serial_type = 5
                # SnapStick0 -> port 0
                serial_port = int(ss_match.groups()[0])
            else:
                com_match = self.com_re.match(self.device)
                if com_match:
                    serial_type = 1
                    # COM1 -> port 0
                    serial_port = int(com_match.groups()[0]) - 1
                else:
                    serial_type = 1
                    serial_port = self.device

        return SCSerialParams(serial_type, serial_port, binascii.unhexlify(self.address) if self.address else None)


class TcpProfile(Profile):

    _spec = [s.strip() for s in """
    kind = option('tcp')
    host = string(default='127.0.0.1')
    port = integer(default=48625)
    ssl = boolean(default=False)
    username = string(default='public')
    password = string(default='public')
    address = string(min=0, max=6, default='')
    network = string(default='default')
    """.split('\n')]

    def __init__(self, name='default'):
        super(TcpProfile, self).__init__(name)

        self.kind = "tcp"
        self.host = "127.0.0.1"
        self.port = 48625
        self.ssl = False
        self.username = "public"
        self.password = "public"
        self.address = ""
        self.network = "default"

    def as_sc_params(self):
        return SCTcpParams(self.host, self.port, self.ssl, self.username, self.password,
                           binascii.unhexlify(self.address) if self.address else None)


class MqttProfile(Profile):

    _spec = [s.strip() for s in """
    kind = option('mqtt')
    host = string(default='127.0.0.1')
    port = integer(default=8883)
    address = string(min=6, max=6, default='000080')
    ca_certs = string()
    username = string()
    password = string()
    network = string(default='default')
    """.split('\n')]

    def __init__(self, name='default'):
        super(MqttProfile, self).__init__(name)
        self.kind = "mqtt"
        self.host = "127.0.0.1"
        self.port = 8883
        self.username = None
        self.password = None
        self.address = '000080'
        self.ca_certs = None
        self.network = 'default'
